unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    Label2: TLabel;
    OutPortEdit: TEdit;
    ConnectButton: TButton;
    MainList: TListBox;
    LatestLabel: TLabel;
    TestTime: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure ConnectClick(Sender: TObject);
    procedure DisconnectClick(Sender: TObject);
    procedure PairTestTimeTimer(Sender: TObject);
  private
    procedure ReadCom(Data: String);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  MTRandomity, Com;

var
  OutCom: TCom;
  PutData: String;
  bGotQuery: Boolean;

function StrToSpacedHex(Text: String): String;
const
  NybbleToHex: array[$0..$F] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := NybbleToHex[Byte(Text[I]) shr 4];
    Result[I * 3 - 1] := NybbleToHex[Byte(Text[I]) and $F];
  end;
end;

function CalcTimestamp: String;
var
  DT: TDateTime;

begin
  DT := Now;
  Result := '[' + DateToStr(DT) + ' ' + TimeToStr(DT) + '] ';
end;

procedure TMainForm.ReadCom(Data: String);
var
  I: Integer;
  bReply: Boolean;

begin
  Data := Trim(Data);
  I := Pos('*', Data);
  if I > 0 then Data := Copy(Data, 1, I-1);
  bReply := True;
       if Data = '$PTNLQBA'       then OutCom.WriteCom('$PTNLRBA,1,1,0*4B'#13#10)
  else if Data = '$PTNLQTP,T'     then OutCom.WriteCom('$PTNLRTP,T,29.07,3.0*27'#13#10)
  else if Data = '$PTNLQVR,S'     then OutCom.WriteCom('$PTNLRVR,S,Aardvark,02.02.00,05,08,2012*37'#13#10)
  else if Data = '$PTNLQVR,H'     then OutCom.WriteCom('$PTNLRVR,H,9005,Aardvark,24005335,08,25,2012,06*22'#13#10)
  else if Data = '$PTNLQDR,STAT1' then OutCom.WriteCom('$PTNLRDR,STAT1,94B,3,0,159746168,1753,200,0.03,0.00,0.000,16*2E'#13#10)
  else if Data = '$PTNLQPT'       then OutCom.WriteCom('$PTNLRPT,019200,8,N,2,C,4*69'#13#10)
  else if Data = '$PTNLQNM'       then OutCom.WriteCom('$PTNLRNM,141,1,1*4F'#13#10)
  else
    bReply := False
  ;
  if bReply then
    MainList.Items.Add(CalcTimestamp + 'Replied to query:')
  else
    MainList.Items.Add(CalcTimestamp + 'Not replying to unknown query:')
  ;
  MainList.Items.Add(Data);

  //MainList.Items.Add(CalcTimestamp + 'End of query sequence.');
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  OutCom := TCom.Create;
  OutCom.OnReadCom := ReadCom;
end;

procedure TMainForm.ConnectClick(Sender: TObject);
const
  MyBaud = 19200;

var
  OutComNum: Integer;
  bOutConnected: Boolean;

begin
  OutComNum := StrToInt(OutPortEdit.Text);
  bOutConnected := OutCom.OpenCom(OutComNum);
  if bOutConnected then begin
    OutCom.SetBaud(MyBaud);
    bGotQuery := False;
    PutData := '';
    TestTime.OnTimer := PairTestTimeTimer;
    TestTime.Interval := 1;
    TestTime.Enabled := True;
    ConnectButton.Caption := 'Disconnect';
    ConnectButton.OnClick := DisconnectClick;
    MainList.Items.Add(CalcTimestamp + 'Connected, testing...');
  end
  else begin
    if bOutConnected then OutCom.CloseCom else MainList.Items.Add('Failed to connect to COM' + IntToStr(OutComNum) + '.');
    MainList.Items.Add('Please check the settings and try again.');
  end;
end;

procedure TMainForm.DisconnectClick(Sender: TObject);
begin
  OutCom.CloseCom;
  TestTime.Enabled := False;
  ConnectButton.Caption := 'Connect';
  ConnectButton.OnClick := ConnectClick;
  MainList.Items.Add(CalcTimestamp + 'Disconnected.');
end;

procedure TMainForm.PairTestTimeTimer(Sender: TObject);
var
  GPSSel: Byte;

begin
  TestTime.Interval := 0;

  LatestLabel.Caption := '';

  GPSSel := RandNybble;

  case GPSSel of
    $0: PutData := '$GPGGA,191152.699,4044.36632,N,07400.10986,W,1,12,0.70,00021.9,M,-034.2,M,,*62'#13#10 + '$GPRMC,191152.699,A,4044.36632,N,07400.10986,W,006.8,116.0,290613,,,A*71'#13#10;
    $1: PutData := '$GPGGA,191151.699,4044.36715,N,07400.11208,W,1,10,0.73,00022.3,M,-034.2,M,,*61'#13#10 + '$GPRMC,191151.699,A,4044.36715,N,07400.11208,W,005.2,116.9,290613,,,A*7A'#13#10;
    $2: PutData := '$GPGGA,191150.699,4044.36781,N,07400.11376,W,1,11,0.98,00022.3,M,-034.2,M,,*61'#13#10 + '$GPRMC,191150.699,A,4044.36781,N,07400.11376,W,003.9,117.6,290613,,,A*7D'#13#10;
    $3: PutData := '$GPGGA,191149.699,4044.36831,N,07400.11501,W,1,11,0.85,00022.3,M,-034.2,M,,*67'#13#10 + '$GPRMC,191149.699,A,4044.36831,N,07400.11501,W,003.7,118.0,290613,,,A*70'#13#10;
    $4: PutData := '$GPGGA,191148.699,4044.36880,N,07400.11621,W,1,11,0.78,00022.3,M,-034.2,M,,*6F'#13#10 + '$GPRMC,191148.699,A,4044.36880,N,07400.11621,W,003.4,118.1,290613,,,A*78'#13#10;
    $5: PutData := '$GPGGA,191147.699,4044.36925,N,07400.11730,W,1,11,0.76,00022.3,M,-034.2,M,,*61'#13#10 + '$GPRMC,191147.699,A,4044.36925,N,07400.11730,W,002.1,118.5,290613,,,A*78'#13#10;
    $6: PutData := '$GPGGA,191146.699,4044.36952,N,07400.11798,W,1,10,0.83,00022.3,M,-034.2,M,,*69'#13#10 + '$GPRMC,191146.699,A,4044.36952,N,07400.11798,W,001.6,118.9,290613,,,A*73'#13#10;
    $7: PutData := '$GPGGA,191145.699,4044.36974,N,07400.11849,W,1,10,0.86,00022.3,M,-034.2,M,,*68'#13#10 + '$GPRMC,191145.699,A,4044.36974,N,07400.11849,W,000.8,120.0,290613,,,A*7A'#13#10;
    $8: PutData := '$GPGGA,191144.699,4044.36986,N,07400.11875,W,1,09,0.86,00022.3,M,-034.2,M,,*63'#13#10 + '$GPRMC,191144.699,A,4044.36986,N,07400.11875,W,000.2,120.9,290613,,,A*7A'#13#10;
    $9: PutData := '$GPGGA,191143.699,4044.36988,N,07400.11880,W,1,10,1.05,00022.3,M,-034.2,M,,*62'#13#10 + '$GPRMC,191143.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*73'#13#10;
    $A: PutData := '$GPGGA,191142.699,4044.36988,N,07400.11880,W,1,09,0.86,00022.3,M,-034.2,M,,*61'#13#10 + '$GPRMC,191142.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*72'#13#10;
    $B: PutData := '$GPGGA,191141.699,4044.36988,N,07400.11880,W,1,12,0.95,00022.3,M,-034.2,M,,*6A'#13#10 + '$GPRMC,191141.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*71'#13#10;
    $C: PutData := '$GPGGA,191140.699,4044.36988,N,07400.11880,W,1,10,0.73,00022.3,M,-034.2,M,,*61'#13#10 + '$GPRMC,191140.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*70'#13#10;
    $D: PutData := '$GPGGA,191139.699,4044.36988,N,07400.11880,W,1,09,0.83,00022.3,M,-034.2,M,,*68'#13#10 + '$GPRMC,191139.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*7E'#13#10;
    $E: PutData := '$GPGGA,191138.699,4044.36988,N,07400.11880,W,1,08,0.88,00022.3,M,-034.2,M,,*63'#13#10 + '$GPRMC,191138.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*7F'#13#10;
    $F: PutData := '$GPGGA,191137.699,4044.36988,N,07400.11880,W,1,10,1.12,00022.3,M,-034.2,M,,*67'#13#10 + '$GPRMC,191137.699,A,4044.36988,N,07400.11880,W,000.0,121.0,290613,,,A*70'#13#10;
  end;
  OutCom.WriteCom(PutData);
  LatestLabel.Caption := PutData;

  TestTime.Interval := 1000;
end;

end.
